<?php
/*
    Plugin Name: BCD Upcoming Posts
    Plugin URI: http://www.duhjones.com/downloads/bcd-upcoming-posts/
    Description: Creates a widget that can be used to display upcoming posts.  It can be customized to display a certain number of posts and in random order.  Also, provides a shortcode to display the list.
    Author: Frank Jones
    Version: 1.1.0
    Author URI: http://www.duhjones.com/
*/

add_action( 'widgets_init', create_function('', 'return register_widget("bcd_upcoming_posts");') );
bcdr_add_action_hooks();


// Add 'init' action hooks
function bcdr_add_action_hooks() {
	add_shortcode( 'bcdupcoming', 'bcdup_sc_upcoming_posts' );
}


class BCD_Upcoming_Posts extends WP_Widget {
	// Initializes the widget frame
	function BCD_Upcoming_Posts() {
		$widget_ops = array(
			'classname' => 'BCD_Upcoming_Posts',
			'description' => 'Displays upcoming posts'
		);
		
		$this->WP_Widget( 'BCD_Upcoming_Posts', 'Upcoming Posts', $widget_ops );
	}
	
	// Defines the options form for the widget
	function form ( $instance ) {
		$instance = wp_parse_args(
			(array) $instance,
			array(
				'title' => '',
				'posts_per_page' => '',
				'random_order' => true
			)
		);
		
		// Title
		$title = $instance['title'];
		$title_field_id = $this->get_field_id('title');
		$title_field_name = $this->get_field_name('title');
		$title_value = attribute_escape($title);
		
		echo '<p>';
		echo $this->build_textbox( $title_field_id, 'Title', 'widefat', $title_field_name, $title_value );
		echo '</p>';
		
		
		// Posts per page
		$posts_per_page = $instance['posts_per_page'];
		$posts_per_page_field_id = $this->get_field_id('posts_per_page');
		$posts_per_page_field_name = $this->get_field_name('posts_per_page');
		$posts_per_page_value = attribute_escape($posts_per_page);

		echo '<p>';
		echo $this->build_textbox( $posts_per_page_field_id, 'Number of posts to show', 'widefat', $posts_per_page_field_name, $posts_per_page_value );
		echo '</p>';
		
		
		// Show in random order
		$random_order = $instance['random_order'];
		$random_order_field_id = $this->get_field_id('random_order');
		$random_order_field_name = $this->get_field_name('random_order');
		$random_order_value = ($instance['random_order'] == 'on') ? true : false;
		
		echo '<p>';
		echo '<input class="checkbox" type="checkbox" ' . checked( $random_order, 'on', false ) . ' id="' . $random_order_field_id . '" name="' . $random_order_field_name . '" /> ';
		echo '<label for="' . $random_order_field_id . '">Display in random order?</label>';
		echo '</p>';
	}
	
	// Overridden from base class
	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;
		$instance['title'] = $new_instance['title'];
		$instance['posts_per_page'] = $new_instance['posts_per_page'];
		$instance['random_order'] = $new_instance['random_order'];
		return $instance;
	}
	
	// Overridden from base class
	function widget( $args, $instance ) {
		extract($args, EXTR_SKIP);
		
		echo $before_widget;
		
		$title = empty($instance['title']) ? ' ' : apply_filters('widget_title', $instance['title']);
		
		if (!empty($title)) {
			echo $before_title . $title . $after_title;;
		}
		
		$qry = 'post_status=future';

		if ( !empty( $instance['posts_per_page'] )) {
			$qry .= '&posts_per_page=' . $instance['posts_per_page'];
		}
		
		if ( !empty( $instance['random_order'] ) && 'on' == $instance['random_order'] ) {
			$qry .= '&orderby=rand';
		}
		
		query_posts( $qry );
		if ( have_posts() ) {
			echo '<ul>';
			while ( have_posts() ) {
				the_post();
				echo '<li><a href="' . get_permalink() . '">' . get_the_title();
				echo the_post_thumbnail( array(220,200) );
				echo '</a></li>';
			}
			echo '</ul>';
		}
		wp_reset_query();
		
		// Widget code goes here
		
		
		echo $after_widget;
	}
	
	// Builds a textbox for display
	function build_textbox( $field_id, $label, $class, $name, $value ) {
		$ret_val = '<label for="%id%">%label%: ';
		$ret_val .= '<input id="%id%" type="text" class="%class%" name="%name%" value="%value%" />';
		$ret_val .= '</label>';
		
		$ret_val = str_replace( '%id%', $field_id, $ret_val );
		$ret_val = str_replace( '%label%', $label, $ret_val );
		$ret_val = str_replace( '%class%', $class, $ret_val );
		$ret_val = str_replace( '%name%', $name, $ret_val );
		$ret_val = str_replace( '%value%', $value, $ret_val );
		
		return $ret_val;
	}
}


function bcdup_sc_upcoming_posts( $atts ) {
	extract( shortcode_atts( array(
		'numposts' => '',
		'random' => 'no',
		'showmore' => 'yes'
	), $atts ) );
	
	$output = '';
	
	global $post;
	$tmp_post = $post;
	
	$qry = 'post_status=future';
	
	if ( !empty( $numposts )) {
		$qry .= '&posts_per_page=' . $numposts;
	}
	
	if ( !empty( $random ) && 'yes' == $random ) {
		$qry .= '&orderby=rand';
	}
	
	query_posts( $qry );
	if ( have_posts() ) {
		while ( have_posts() ) {
			the_post();
			$output .= '<div>';
			$output .= '<strong><a href="' . get_permalink() . '">' . get_the_title() . '</a></strong>';
			if ( !empty( $showmore ) && 'yes' == $showmore ) {
				$output .= '<p>' . get_the_excerpt(__('(more)')) . '</p>';
			}
			$output .= '</div>';
		}
	}
	wp_reset_query();

	$output .= '';
	
	return $output;
}

?>
